import { env } from 'bun';

type SelectBookmark = {
  id: number;
  name: string;
  href: string;
  finished: boolean;
  created_at: number;
  updated_at: number;
};

const baseUrl = env.API_SERVICE_URL;
const bookmarksUrl = `${baseUrl}/bookmarks`;

const noBodyHeaders = new Headers({
  Authorization: `Bearer ${env.BEARER_TOKEN}`,
});

const bodyHeaders = new Headers({
  'Content-Type': 'application/json',
  Authorization: `Bearer ${env.BEARER_TOKEN}`,
});

async function query(): Promise<Array<SelectBookmark>> {
  try {
    const response = await fetch(bookmarksUrl, {
      method: 'GET',
      signal: AbortSignal.timeout(50000),
      headers: noBodyHeaders,
    });

    if (!response.ok) {
      throw new Error('Failed to fetch bookmarks');
    }

    const data = await response.json();
    if (!Array.isArray(data)) {
      throw new Error('Invalid bookmarks data');
    }

    return data;
  } catch (err) {
    console.error(err);
    throw new Error('Failed to fetch bookmarks');
  }
}

async function remove(id: number) {
  try {
    const response = await fetch(`${bookmarksUrl}/${id}`, {
      method: 'DELETE',
      signal: AbortSignal.timeout(5000),
      headers: noBodyHeaders,
    });

    if (!response.ok) {
      throw new Error('Failed to remove bookmark');
    }
  } catch (err) {
    console.error(err);
    throw new Error('Failed to remove bookmark');
  }
}

async function update(id: number, body: { name: string; href: string }) {
  try {
    const response = await fetch(`${bookmarksUrl}/${id}`, {
      method: 'PUT',
      body: JSON.stringify(body),
      signal: AbortSignal.timeout(5000),
      headers: bodyHeaders,
    });

    if (!response.ok) {
      throw new Error('Failed to update bookmark');
    }
  } catch (err) {
    console.error(err);
    throw new Error('Failed to update bookmark');
  }
}

async function check(id: number, finished: boolean) {
  try {
    const response = await fetch(`${bookmarksUrl}/${id}/check/${finished}`, {
      method: 'PUT',
      signal: AbortSignal.timeout(5000),
      headers: noBodyHeaders,
    });
    if (!response.ok) {
      throw new Error('Failed to check bookmark');
    }
  } catch (err) {
    console.error(err);
    throw new Error('Failed to check bookmark');
  }
}

async function create(body: { name: string; href: string }) {
  try {
    const response = await fetch(`${bookmarksUrl}`, {
      method: 'POST',
      body: JSON.stringify(body),
      signal: AbortSignal.timeout(5000),
      headers: bodyHeaders,
    });
    if (!response.ok) {
      throw new Error('Failed to create bookmark');
    }
  } catch (err) {
    if (err) {
      console.error(err);
      throw new Error('Failed to create bookmark');
    }
  }
}

export default { query, remove, update, check, create };
